@@ -7,21 +7,26 @@ package JSON::Tiny;
 
 use strict;
 use warnings;
+use utf8;
 use B;
-use Carp 'croak';
+use Carp qw/carp croak/;
 use Exporter 'import';
 use Scalar::Util 'blessed';
 use Encode ();
 
-our $VERSION = '0.50';
-our @EXPORT_OK = qw(decode_json encode_json j);
+our $VERSION = '0.51';
+our @EXPORT_OK = qw(decode_json encode_json false from_json j to_json true);
 
 # Constructor and error inlined from Mojo::Base
+# DEPRECATED in 0.51.
 sub new {
+  local $Carp::CarpLevel = 1;
+  carp( 'Object-oriented JSON::Tiny API is DEPRECATED' );
   my $class = shift;
   bless @_ ? @_ > 1 ? {@_} : {%{$_[0]}} : {}, ref $class || $class;
 }
 
+# DEPRECATED IN 0.51.
 sub error {
   return $_[0]{error} if @_ == 1;
   $_[0]{error} = $_[1];
@@ -30,8 +35,7 @@ sub error {
 
 # Literal names
 # Users may override Booleans with literal 0 or 1 if desired.
-our $FALSE = bless \(my $false = 0), 'JSON::Tiny::_Bool';
-our $TRUE  = bless \(my $true  = 1), 'JSON::Tiny::_Bool';
+our($FALSE, $TRUE) = map { bless \(my $dummy = $_), 'JSON::Tiny::_Bool' } 0, 1;
 
 # Escaped special character map with u2028 and u2029
 my %ESCAPE = (
@@ -54,52 +58,58 @@ for(0x00 .. 0x1f) {
     if ! defined $REVERSE{$packed};
 }
 
+# DEPRECATED IN 0.51.
 sub decode {
-  my $self = shift->error(undef);
-  my $value;
-  return $value if eval{ $value = _decode(shift); 1 };
-  $self->error(_chomp($@));
-  return undef;  ## no critic(return)
+  shift->error(my $err = _decode(\my $value, pop));
+  return defined $err ? undef : $value;
 }
 
 sub decode_json {
-  my $value;
-  return eval { $value = _decode(shift); 1 } ? $value : croak _chomp($@);
+  my $err = _decode(\my $value, shift);
+  return defined $err ? croak $err : $value;
 }
 
+# DEPRECATED IN 0.51.
 sub encode { encode_json($_[1]) }
 
 sub encode_json { Encode::encode 'UTF-8', _encode_value(shift); }
 
-sub false {$FALSE}
+sub false () {$FALSE}  ## no critic (prototypes)
+
+sub from_json {
+  my $err = _decode(\my $value, shift, 1);
+  return defined $err ? croak $err : $value;
+}
 
 sub j {
   return encode_json $_[0] if ref $_[0] eq 'ARRAY' || ref $_[0] eq 'HASH';
   return decode_json $_[0];
 }
 
-sub true {$TRUE}
+sub to_json { _encode_value(shift) }
 
-sub _chomp { chomp $_[0] ? $_[0] : $_[0] }
+sub true () {$TRUE} ## no critic (prototypes)
 
 sub _decode {
-  # Missing input
-  die "Missing or empty input\n" unless length(local $_ = shift);
+  my $valueref = shift;
 
-  # Wide characters
-  die "Wide character in input\n" unless utf8::downgrade($_, 1);
+  eval {
 
-  # UTF-8
-  die "Input is not UTF-8 encoded\n"
-    unless eval { $_ = Encode::decode('UTF-8', $_, 1); 1 };
+    # Missing input
+    die "Missing or empty input\n" unless length( local $_ = shift );
 
-  # Value
-  my $value = _decode_value();
+    # UTF-8
+    $_ = eval { Encode::decode('UTF-8', $_, 1) } unless shift;
+    die "Input is not UTF-8 encoded\n" unless defined $_;
+
+    # Value
+    $$valueref = _decode_value();
   
-  # Leftover data
-  _exception('Unexpected data') unless m/\G[\x20\x09\x0a\x0d]*\z/gc;
+    # Leftover data
+    return m/\G[\x20\x09\x0a\x0d]*\z/gc || _throw('Unexpected data');
+  } ? return undef : chomp $@;
 
-  return $value;
+  return $@;
 }
 
 sub _decode_array {
@@ -116,7 +126,7 @@ sub _decode_array {
     last if m/\G[\x20\x09\x0a\x0d]*\]/gc;
 
     # Invalid character
-    _exception('Expected comma or right square bracket while parsing array');
+    _throw('Expected comma or right square bracket while parsing array');
   }
 
   return \@array;
@@ -128,14 +138,14 @@ sub _decode_object {
 
     # Quote
     m/\G[\x20\x09\x0a\x0d]*"/gc
-      or _exception('Expected string while parsing object');
+      or _throw('Expected string while parsing object');
 
     # Key
     my $key = _decode_string();
 
     # Colon
     m/\G[\x20\x09\x0a\x0d]*:/gc
-      or _exception('Expected colon while parsing object');
+      or _throw('Expected colon while parsing object');
 
     # Value
     $hash{$key} = _decode_value();
@@ -147,7 +157,7 @@ sub _decode_object {
     last if m/\G[\x20\x09\x0a\x0d]*\}/gc;
 
     # Invalid character
-    _exception('Expected comma or right curly bracket while parsing object');
+    _throw('Expected comma or right curly bracket while parsing object');
   }
 
   return \%hash;
@@ -162,9 +172,9 @@ sub _decode_string {
 
   # Invalid character
   unless (m/\G"/gc) {
-    _exception('Unexpected character or invalid escape while parsing string')
+    _throw('Unexpected character or invalid escape while parsing string')
       if m/\G[\x00-\x1f\\]/;
-    _exception('Unterminated string');
+    _throw('Unterminated string');
   }
 
   # Unescape popular characters
@@ -190,11 +200,11 @@ sub _decode_string {
 
         # High surrogate
         ($ord & 0xfc00) == 0xd800
-          or pos($_) = $pos + pos($str), _exception('Missing high-surrogate');
+          or pos($_) = $pos + pos($str), _throw('Missing high-surrogate');
 
         # Low surrogate
         $str =~ m/\G\\u([Dd][C-Fc-f]..)/gc
-          or pos($_) = $pos + pos($str), _exception('Missing low-surrogate');
+          or pos($_) = $pos + pos($str), _throw('Missing low-surrogate');
 
         $ord = 0x10000 + ($ord - 0xd800) * 0x400 + (hex($1) - 0xdc00);
       }
@@ -236,12 +246,11 @@ sub _decode_value {
   return undef if m/\Gnull/gc;  ## no critic (return)
 
   # Invalid character
-  _exception('Expected string, array, object, number, boolean or null');
+  _throw('Expected string, array, object, number, boolean or null');
 }
 
 sub _encode_array {
-  my $array = shift;
-  '[' . join(',', map { _encode_value($_) } @$array) . ']';
+  '[' . join(',', map { _encode_value($_) } @{$_[0]}) . ']';
 }
 
 sub _encode_object {
@@ -292,7 +301,7 @@ sub _encode_value {
   return _encode_string($value);
 }
 
-sub _exception {
+sub _throw {
 
   # Leading whitespace
   m/\G[\x20\x09\x0a\x0d]*/gc;
@@ -310,6 +319,5 @@ sub _exception {
 
 # Emulate boolean type
 package JSON::Tiny::_Bool;
-use overload '0+' => sub { ${$_[0]} }, '""' => sub { ${$_[0]} }, fallback => 1;
-
+use overload '""' => sub { ${$_[0]} }, fallback => 1;
 1;
